unit ModemEmulator;

{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls{, Windows};

type
  TFormModemEmulator = class(TForm)
    Panel3: TPanel;
    Button2: TSpeedButton;
    Button4: TSpeedButton;
    Button5: TSpeedButton;
    Button3: TSpeedButton;
    Button1: TSpeedButton;
    Button6: TSpeedButton;
    Button8: TSpeedButton;
    ButtonStar: TSpeedButton;
    Button0: TSpeedButton;
    Button9: TSpeedButton;
    Button7: TSpeedButton;
    ButtonHash: TSpeedButton;
    Panel1: TPanel;
    ButtonCall: TButton;
    ButtonHangup: TButton;
    MemoOutput: TMemo;
    procedure ButtonCallClick(Sender: TObject);
    procedure ButtonHangupClick(Sender: TObject);
    procedure ButtonDigitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FLine: integer;
    FOffHook: boolean;
    FRingCount: integer;
    FRingLimit: integer;
    FDigitBuffer: string;
    FOnDigit: TNotifyEvent;
    FOnCallIncoming: TNotifyEvent;
    FOnRing: TNotifyEvent;
    FAbort: boolean;
    FBusyCount: integer;
    FPendingAbort: integer;
    FOnHangup: TNotifyEvent;
    FLastDigit: char;
  protected
    procedure EmulateHangupDetect;
    procedure EmulateRingDetect;
    procedure EmulateDigitDetect(Digit: char);
    procedure CheckLine;
    procedure CheckOffHook;
    procedure CheckOnHook;
    procedure BeginBusy;
    procedure EndBusy;
  public
    procedure AllocateLine(Line: integer);
    procedure ReleaseLine;
    procedure Reset;
    procedure Stop;
    procedure DetectRing(Count: integer = 1);
    procedure Pickup;
    procedure Hangup;
    procedure Purge;
    procedure Speak(const Phrase: string; const TerminatingDigits: string = '');
    function GetDigits(const AllowedDigits: string = ''; const TerminatingDigits: string = ''; Timeout: integer = -1): string;
    // function RecordMessage(MaxLength: integer; const TerminatingDigits: string): string;

    property OnRing: TNotifyEvent read FOnRing write FOnRing;
    property OnDigit: TNotifyEvent read FOnDigit write FOnDigit;
    property OnCallIncoming: TNotifyEvent read FOnCallIncoming write FOnCallIncoming;
    property OnHangup: TNotifyEvent read FOnHangup write FOnHangup;
    property LastDigit: char read FLastDigit;
    property OffHook: boolean read FOffHook;
  end;

var
  FormModemEmulator: TFormModemEmulator;

implementation

{$R *.lfm}

procedure TFormModemEmulator.FormCreate(Sender: TObject);
begin
  FLine := -1;
end;

procedure TFormModemEmulator.AllocateLine(Line: integer);
begin
  ReleaseLine;
  ButtonCall.Enabled := True;
  ButtonHangup.Enabled := False;
  FLine := Line;
end;

const
  sDigits = '0123456789*#';

procedure TFormModemEmulator.ButtonDigitClick(Sender: TObject);
begin
  EmulateDigitDetect(sDigits[TSpeedButton(Sender).Tag+1]);
end;

procedure TFormModemEmulator.BeginBusy;
begin
  inc(FBusyCount);
  if (FAbort) then
    inc(FPendingAbort);
end;

procedure TFormModemEmulator.ButtonCallClick(Sender: TObject);
begin
  EmulateRingDetect;
  ButtonCall.Enabled := False;
  ButtonHangup.Enabled := True;
end;

procedure TFormModemEmulator.ButtonHangupClick(Sender: TObject);
begin
  EmulateHangupDetect;
  ButtonCall.Enabled := True;
  ButtonHangup.Enabled := False;
end;

procedure TFormModemEmulator.CheckLine;
begin
  if (FLine = -1) then
    raise Exception.Create('Line has not been allocated');
end;

procedure TFormModemEmulator.CheckOffHook;
begin
  if (not FOffHook) then
    raise Exception.Create('Phone is not off hook');
end;

procedure TFormModemEmulator.CheckOnHook;
begin
  if (FOffHook) then
    raise Exception.Create('Phone is not on hook');
end;

procedure TFormModemEmulator.DetectRing(Count: integer);
begin
  CheckLine;
  CheckOnHook;
  FRingLimit := Count;
  FRingCount := 0;
end;

procedure TFormModemEmulator.EmulateDigitDetect(Digit: char);
begin
  if (FLine = -1) then
    exit;

  FDigitBuffer := FDigitBuffer+Digit;

  MemoOutput.Lines.Add(Format('[digit:%s]', [Digit]));

  if (Assigned(OnDigit)) then
    OnDigit(Self);
end;

procedure TFormModemEmulator.EmulateHangupDetect;
begin
  if (FLine = -1) or (not FOffHook) then
    exit;

  FOffHook := False;
  FRingCount := 0;
  FLastDigit := #0;

  MemoOutput.Lines.Add('[hangup]');

  if (Assigned(OnHangup)) then
    OnHangup(Self);
end;

procedure TFormModemEmulator.EmulateRingDetect;
begin
  if (FLine = -1) then
    exit;
  CheckOnHook;
  inc(FRingCount);

  MemoOutput.Lines.Add(format('[ring: %d]', [FRingCount]));

  if (Assigned(OnRing)) then
    OnRing(Self);

  if (FRingCount >= FRingLimit) and (Assigned(OnCallIncoming)) then
    OnCallIncoming(Self);
end;

procedure TFormModemEmulator.EndBusy;
begin
  dec(FBusyCount);

  if (FAbort) then
  begin
    dec(FPendingAbort);
    if (FPendingAbort = 0) then
      FAbort := False;
  end;
end;

function TFormModemEmulator.GetDigits(const AllowedDigits: string;
  const TerminatingDigits: string; Timeout: integer): string;
var
  Digit: char;
  StartTime: DWORD;
  GotT1000: boolean;
  i: integer;
begin
  CheckLine;
  CheckOffHook;
  Result := '';

  StartTime := GetTickCount;

  BeginBusy;
  try

    GotT1000 := False;

    while ((Result = '') or (TerminatingDigits <> '')) and
      (not FAbort) and (FOffHook) and (not GotT1000) do
    begin
      i := 1;
      while (i <= Length(FDigitBuffer)) do
      begin
        Digit := FDigitBuffer[i];

        if (AllowedDigits = '') or (Pos(Digit, AllowedDigits) > 0) then
        begin
          MemoOutput.Lines.Add(Format('[get:%s]', [Digit]));
          Result := Result+Digit;
          FLastDigit := Digit;
        end;

        if (Pos(Digit, TerminatingDigits) > 0) then
        begin
          GotT1000 := True;
          FDigitBuffer := Copy(FDigitBuffer, i+1, Length(FDigitBuffer)-i-1);
          FLastDigit := Digit;
          break;
        end;
        inc(i);
      end;

      if (not GotT1000) then
        FDigitBuffer := '';
      Application.ProcessMessages;

      if (Timeout <> -1) and (GetTickCount-StartTime > Timeout) then
        break;
    end;


  finally
    EndBusy;
  end;
end;

procedure TFormModemEmulator.Hangup;
begin
  CheckLine;
  Stop;
  FOffHook := False;
  FRingCount := 0;
  FLastDigit := #0;
end;

procedure TFormModemEmulator.Pickup;
begin
  CheckLine;
  FOffHook := True;
end;

procedure TFormModemEmulator.Purge;
begin
  CheckLine;
  FDigitBuffer := '';
  FLastDigit := #0;
  MemoOutput.Clear;
end;

procedure TFormModemEmulator.ReleaseLine;
begin
 if (FLine <> -1) then
    Reset;
  FLine := -1;
  ButtonCall.Enabled := False;
  ButtonHangup.Enabled := False;
end;

procedure TFormModemEmulator.Reset;
begin
  CheckLine;

  BeginBusy;
  try
    Hangup;
    Purge;
    FRingCount := 0;
    MemoOutput.Clear;
  finally
    EndBusy;
  end;
end;

procedure TFormModemEmulator.Speak(const Phrase, TerminatingDigits: string);
begin
  CheckLine;
  CheckOffHook;

  BeginBusy;
  try
    MemoOutput.Lines.Add(Phrase);
  finally
    EndBusy;
  end;
end;

procedure TFormModemEmulator.Stop;
begin
  if (not FAbort) then
    FPendingAbort := FBusyCount;
  FAbort := True;
end;

end.
